from midi.MidiInFile import MidiInFile
from midi.MidiOutStream import MidiOutStream

CTRLR_EVENTS = {1: 'modulation',
                2: 'breath',
                4: 'foot_pedal',
                5: 'portamento_time',
                7: 'volume',
                8: 'balance',
                10: 'pan',
                11: 'expression',
                64: 'sustain',
                71: 'resonance',
                74: 'brightness',
                91: 'reverb',
                92: 'tremolo',
                93: 'chorus',
                94: 'detune',
                95: 'phaser'}
class MidiHandler(MidiOutStream):
    def __init__(self):
        MidiOutStream.__init__(self)
        self.channels = {}
        self.ending = 0
        self.msb = 0
        self.tpb = 0
    def _expand_channels(self, channel):
        if not self.channels.has_key(channel):
            self.channels[channel] = {}
    def note_on(self, channel=0, note=0x40, velocity=0x40):
        self._expand_channels(channel)
        if not self.channels[channel].has_key('notes'):
            self.channels[channel]['notes'] = []
        self.channels[channel]['notes'] += [[note, velocity, self.abs_time(), 0]]
    def note_off(self, channel=0, note=0x40, velocity=0x40):
        for i in range(len(self.channels[channel]['notes'])-1, -1, -1):
            # we want to search backwards
            if self.channels[channel]['notes'][i][0] == note:
                self.channels[channel]['notes'][i][3] = self.abs_time()
                break
    def eof(self):
        self.ending = self.abs_time()
    def instrument_name(self, text):
        self._expand_channels(self.get_current_track())
        self.channels[self.get_current_track()]['instrument_name'] = text
    def sequence_name(self, text):
        self._expand_channels(self.get_current_track())
        self.channels[self.get_current_track()]['sequence_name'] = text
    def pitch_bend(self, channel, value):
        self._expand_channels(channel)
        if not self.channels[channel].has_key('pitch'):
            self.channels[channel]['pitch'] = []
        self.channels[channel]['pitch'] += [(value, self.abs_time())]
    def channel_pressure(self, channel, pressure):
        self._expand_channels(channel)
        if not self.channels[channel].has_key('pressure'):
            self.channels[channel]['pressure'] = []
        self.channels[channel]['pressure'] += [(value, self.abs_time())]
    def continuous_controller(self, channel, controller, value):
        self._expand_channels(channel)
        if CTRLR_EVENTS.has_key(controller):
            name = CTRLR_EVENTS[controller]
        else:
            sys.stderr.write("Warning: No such controller event %i (on channel %i)\n" % (controller, channel))
            return
        if not self.channels[channel].has_key(name):
            self.channels[channel][name] = []
        self.channels[channel][name] += [(value, self.abs_time())]
    def tempo(self, value):
        self.usb = value
    def time_signature(self, nn, dd, cc, bb):
        self.tpb = cc
    def sysex_event(self, data):
        pass

if __name__ == "__main__":
    import sys
    if len(sys.argv) < 2:
        sys.stderr.write("Usage:\n")
        sys.stderr.write("python midi2inc.py infile [outfile]\n")
        sys.exit(1)
    m = MidiHandler()
    MidiInFile(m, sys.argv[1]).read()
    if len(sys.argv) > 2:
        f = open(sys.argv[2], 'w')
    else:
        f = open(sys.argv[1][:sys.argv[1].rfind('.')]+'.inc', 'w')
    f.write("#declare ending = %i;\n" % m.ending)
    f.write("#declare us_per_beat = %i;\n" % m.usb)
    f.write("#declare ticks_per_beat = %i;\n" % m.tpb)
    for idx, track in m.channels.iteritems():
        if not track.has_key('notes'):
            continue
        if track.has_key('instrument_name'):
            f.write('// Instrument name: ')
            f.write(track['instrument_name'])
            f.write('\n')
        if track.has_key('sequence_name'):
            f.write('// Sequence name: ')
            f.write(track['sequence_name'])
            f.write('\n')
        f.write('#declare track_%i_notes = array[%i] {' % (idx, len(track['notes'])))
        for note in track['notes']:
            f.write('\n\t<%r, %r, %r, %r>,' % tuple(note))
        f.write('\n}\n')
        for ctrlr, vals in track.iteritems():
            if ctrlr in ('instrument_name', 'sequence_name', 'notes'):
                continue
            f.write('#declare track_%i_%s = spline {' % (idx, ctrlr))
            track[ctrlr] = [(64, 0)]+track[ctrlr]
            for i, (value, time) in enumerate(track[ctrlr]):
                if i > 0:
                    f.write('\n\t%s, %r,' % (time-0.01, track[ctrlr][i-1][0]))
                f.write('\n\t%r, %r,' % (time, value))
            f.write('\n}\n')
        f.write('\n')
        sys.stderr.write("track %i minimum: %i\n" % (idx, min(a[0] for a in track['notes'])))
        sys.stderr.write("track %i maximum: %i\n" % (idx, max(a[0] for a in track['notes'])))
    f.close()
